@@ -35,7 +35,7 @@ module MsgCab
       end
 
       def update(props, numbers)
-        numbers = (0 .. CLI.database.max).to_a if numbers.empty?
+        numbers = (0 .. CLI.database.max).to_a if numbers.nil?
         pbar = ProgressBar.new('property', numbers.length) unless opts[:quiet]
         rules = Hash.new
         props.each do |prop|
