@@ -128,37 +128,6 @@ end
 
 module MsgCab
   class Database
-    def update_parent_1(result, msg_id, ref_ids, unresolved,
-                        exclude = Array.new)
-      hash = Hash.new
-      ref_ids.each do |ref_id|
-        gref_ids = ancestors(ref_id)
-        if gref_ids.empty?
-          unresolved << ref_id
-        else
-          hash[ref_id] = gref_ids
-          result.delete_if do |r|
-            gref_ids.include?(r)
-          end
-          break result if result.length <= 1
-        end
-      end
-
-      exclude |= ref_ids
-      if result.length > 1
-        ref_ids.each do |ref_id|
-          if hash[ref_id] && hash[ref_id].length > 0
-	    gref_ids = hash[ref_id] - exclude
-            update_parent_1(result, ref_id, gref_ids, unresolved,
-                           exclude)
-            break if result.length <= 1
-	    exclude |= gref_ids
-          end
-        end
-      end
-      result
-    end
-
     def update_parent(msg_id)
       if @adapter.select_one(<<'End', msg_id)
 SELECT * FROM parent WHERE msg_id = ?
@@ -166,37 +135,49 @@ End
         log(3, "#{msg_id} is already registered as parent")
         return
       end
-      ref_ids = ancestors(msg_id)
-      result = ref_ids.dup
-      unresolved = Array.new
-      update_parent_1(result, msg_id, ref_ids, unresolved)
-      unresolved.uniq!
-      unresolved_ids =
+
+      not_arrived_msg_ids =
         @adapter.select_all(<<'End', msg_id).collect {|row| row[0]}
-SELECT msg_id FROM unresolved_parent WHERE ref_id = ?
+SELECT msg_id FROM not_arrived WHERE ref_id = ?
 End
       @adapter.execute(<<'End', msg_id)
-DELETE FROM unresolved_parent WHERE ref_id = ?
+DELETE FROM not_arrived WHERE ref_id = ?
 End
-      unresolved_ids.each do |ref_id|
-        update_parent(ref_id)
+      not_arrived_msg_ids.each do |not_arrived_msg_id|
+        update_parent(not_arrived_msg_id)
       end
-      if result.length <= 1
-        @adapter.execute(<<'End', msg_id)
-DELETE FROM unresolved_parent WHERE msg_id = ?
+
+      not_arrived_ref_ids = Array.new
+      stmt = @adapter.prepare(<<'End')
+SELECT * FROM message WHERE msg_id = ?
 End
-        if result.length == 1
-          @adapter.execute(<<'End', msg_id, result[0])
+      ref_ids = ancestors(msg_id)
+      ref_ids.dup.each do |ref_id|
+        stmt.execute(ref_id)
+        if stmt.fetch
+          ref_ids -= ancestors(ref_id)
+          break if ref_ids.length <= 1
+        else
+          not_arrived_ref_ids << ref_id
+        end
+      end
+      if ref_ids.length <= 1 || not_arrived_ref_ids.empty?
+        @adapter.execute(<<'End', msg_id)
+DELETE FROM not_arrived WHERE msg_id = ?
+End
+      end
+      if ref_ids.length <= 1
+        parent = ref_ids.length == 1 ? ref_ids[0] : msg_id
+        @adapter.execute(<<'End', msg_id, parent)
 INSERT INTO parent VALUES (?, ?)
 End
-        end
       else
-        unresolved.each do |ref_id|
-          unless @adapter.select_one(<<'End', msg_id, ref_id)
-SELECT * FROM unresolved_parent WHERE msg_id = ? AND ref_id = ?
+        not_arrived_ref_ids.each do |not_arrived_ref_id|
+          unless @adapter.select_one(<<'End', msg_id, not_arrived_ref_id)
+SELECT * FROM not_arrived WHERE msg_id = ? AND ref_id = ?
 End
-            @adapter.execute(<<'End', msg_id, ref_id)
-INSERT INTO unresolved_parent VALUES (?, ?)
+            @adapter.execute(<<'End', msg_id, not_arrived_ref_id)
+INSERT INTO not_arrived VALUES (?, ?)
 End
           end
         end
