@@ -81,10 +81,10 @@ CREATE TABLE parent (
 
 CREATE INDEX idx_parent_ref_id ON parent(ref_id);
 
-CREATE TABLE unresolved_parent (
+CREATE TABLE not_arrived (
 	msg_id varchar(255) NOT NULL,
 	ref_id varchar(255) NOT NULL
 );
 
-CREATE INDEX idx_unresolved_parent_msg_id ON unresolved_parent(msg_id);
-CREATE INDEX idx_unresolved_parent_ref_id ON unresolved_parent(ref_id);
+CREATE INDEX idx_not_arrived_msg_id ON not_arrived(msg_id);
+CREATE INDEX idx_not_arrived_ref_id ON not_arrived(ref_id);
