@@ -21,27 +21,28 @@ require 'msgcab/webapp/view'
 
 module MsgCab
   module WebApp
-    class InvalidRequest < StandardError; end
+    class Error < StandardError; end
+    class InvalidRequest < Error; end
 
     class Controller
-      @@rules = Array.new
+      def initialize
+        @actions = Array.new
+      end
 
-      def self.define_rule(name, pat, *capture_names)
-        @@rules.unshift([name, pat, *capture_names])
+      def add_action(pat, function = nil, &block)
+        raise InvalidArgument unless function || block_given?
+        @actions.unshift([pat, function || block])
       end
 
       def handle_request(webapp)
         begin
           path = (webapp.path_info || '').sub(%r|\A/+|, '').sub(%r|/+\z|, '')
-          rule = @@rules.detect {|name, pat, *capture_names| path =~ pat}
-          unless rule
+          action = @actions.detect {|pat, block| path =~ pat}
+          unless action
             raise InvalidRequest, 'unknown request'
           end
-          unless respond_to?("do_#{rule[0]}")
-            raise InvalidRequest, "unknown request: #{rule[0]}"
-          end
-          send("do_#{rule[0]}", webapp, *$~.captures)
-        rescue InvalidRequest => e
+          action[1].call(webapp, *$~.captures)
+        rescue Error => e
           view = View.new(webapp)
           view.attributes[:cause] = e
           view.output_template('error.html')
