@@ -22,58 +22,54 @@ require 'msgcab/mailtree'
 require 'msgcab/webapp/msgcab_view'
 require 'enumerator'
 
+module MsgCab::WebApp
+  FolderPat = %r|([a-z0-9-]+)|
+end
+  
 class MsgCab::WebApp::MsgCabController < MsgCab::WebApp::Controller
   include MsgCab
   include MsgCab::WebApp
   include Logging
 
-  FolderPat = %r|([a-z0-9-]+)|
+  def initialize
+    super
 
-  define_rule(:archive, /\A\z/)
+    add_action(/\A\z/,
+               method(:do_archive).to_proc)
 
-  define_rule(:index,
-              %r|\A#{FolderPat}/-\z|,
-              :folder_name)
+    add_action(%r|\A#{FolderPat}/-\z|,
+               method(:do_index).to_proc)
 
-  define_rule(:recent_summary,
-              %r|\A#{FolderPat}(?:/:(\d+))?\z|,
-              :folder_name, :max)
+    add_action(%r|\A#{FolderPat}(?:/:(\d+))?\z|,
+               method(:do_recent_summary).to_proc)
 
-  define_rule(:recent_summary_thread,
-              %r|\A#{FolderPat}/thread(?:/:(\d+))?\z|,
-              :folder_name, :max)
+    add_action(%r|\A#{FolderPat}/thread(?:/:(\d+))?\z|,
+               method(:do_recent_summary_thread).to_proc)
 
-  define_rule(:number_summary,
-              %r|\A#{FolderPat}/(\d+)-(\d+)\z|,
-              :folder_name, :min, :max)
+    add_action(%r|\A#{FolderPat}/(\d+)-(\d+)\z|,
+               method(:do_number_summary).to_proc)
 
-  define_rule(:number_summary_mbox,
-              %r|\A#{FolderPat}/(\d+)-(\d+)\.txt\z|,
-              :folder_name, :min, :max)
+    add_action(%r|\A#{FolderPat}/(\d+)-(\d+)\.txt\z|,
+               method(:do_number_summary_mbox).to_proc)
 
-  define_rule(:number_summary_thread,
-              %r|\A#{FolderPat}/thread/(\d+)-(\d+)\z|,
-              :folder_name, :min, :max)
+    add_action(%r|\A#{FolderPat}/thread/(\d+)-(\d+)\z|,
+               method(:do_number_summary_thread).to_proc)
 
-  define_rule(:entity,
-              %r|\A#{FolderPat}/(\d+)(/\d+(?:\.\d+)+)?\z|,
-              :folder_name, :folder_number, :cid)
+    add_action(%r|\A#{FolderPat}/(\d+)(/\d+(?:\.\d+)+)?\z|,
+               method(:do_entity).to_proc)
 
-  define_rule(:entity_mbox,
-              %r|\A#{FolderPat}/(\d+)(/\d+(?:\.\d+)+)?\.txt\z|,
-              :folder_name, :folder_number, :cid)
+    add_action(%r|\A#{FolderPat}/(\d+)(/\d+(?:\.\d+)+)?\.txt\z|,
+               method(:do_entity_mbox).to_proc)
 
-  define_rule(:threads,
-              %r|\A#{FolderPat}/(\d+)\.threads\z|,
-              :folder_name, :folder_number)
+    add_action(%r|\A#{FolderPat}/(\d+)\.threads\z|,
+               method(:do_threads).to_proc)
 
-  define_rule(:js,
-              %r|\A#{FolderPat}/(\d+)\.js|,
-              :folder_name, :folder_number)
+    add_action(%r|\A#{FolderPat}/(\d+)\.js|,
+               method(:do_js).to_proc)
 
-  define_rule(:data,
-              %r|\A\.data(?:/([a-z]+))?/([^\\/]+\.[^\\/.]+)\z|,
-              :suffix, :filename)
+    add_action(%r|\A\.data(?:/([a-z]+))?/([^\\/]+\.[^\\/.]+)\z|,
+               method(:do_data).to_proc)
+  end
 
   private
   def thread_runs(threads)
@@ -144,11 +140,9 @@ class MsgCab::WebApp::MsgCabController < MsgCab::WebApp::Controller
     runs = thread_runs(threads)
     threads.each_with_index do |thread, index|
       thread.xref.sort! {|a, b| a[0] <=> b[0]}
-      cur = thread.xref.assoc(folder.name)
-      if cur
-        thread.xref.delete_at(thread.xref.index(cur))
-        thread.xref.unshift(cur)
-      end
+      thread.xref, other =
+        thread.xref.partition {|xref| xref[0] == folder.name}
+      thread.xref.concat(other)
       thread.nov = thread.nov.decode(encoding)
       thread.icons = thread_icons(runs, threads, thread.depth, index)
       thread.from = ellipsis(thread.nov.canonical_from,
@@ -158,6 +152,7 @@ class MsgCab::WebApp::MsgCabController < MsgCab::WebApp::Controller
     end
   end
 
+  public
   def database
     @database ||= Database.instance
   end
@@ -166,7 +161,6 @@ class MsgCab::WebApp::MsgCabController < MsgCab::WebApp::Controller
     @mailtree ||= MailTree.new
   end
 
-  public
   def do_data(webapp, suffix, filename)
     filename.untaint
     if suffix
