@@ -35,7 +35,7 @@ class FilterRule
     unless folder_number
       folder_number, = fields['x-mail-count']
     end
-    return "unknown:#{number}" unless folder_number
+    return "#{folder_name}:#{number}" unless folder_number
     "#{folder_name}:#{folder_number}"
   end
 end
